@@ -75,7 +75,6 @@ import com.openexchange.java.Strings;
 import com.openexchange.oauth.API;
 import com.openexchange.oauth.OAuthAccount;
 import com.openexchange.oauth.OAuthExceptionCodes;
-import com.openexchange.oauth.OAuthUtil;
 import com.openexchange.session.Session;
 
 /**
@@ -106,7 +105,6 @@ final class DropboxExceptionHandler {
 
         // Invalid token or account was unlinked
         if (InvalidAccessTokenException.class.isInstance(e)) {
-            String cburl = OAuthUtil.buildCallbackURL(oauthAccount);
             API api = oauthAccount.getAPI();
             return OAuthExceptionCodes.OAUTH_ACCESS_TOKEN_INVALID.create(api.getName(), oauthAccount.getId(), session.getUserId(), session.getContextId());
         }
